package sun.net.idn;

import sun.text.normalizer.UCharacter;
import java.text.ParseException;
import sun.text.normalizer.UTF16;

public final class Punycode
{
    private static final int BASE = 36;
    private static final int TMIN = 1;
    private static final int TMAX = 26;
    private static final int SKEW = 38;
    private static final int DAMP = 700;
    private static final int INITIAL_BIAS = 72;
    private static final int INITIAL_N = 128;
    private static final int HYPHEN = 45;
    private static final int DELIMITER = 45;
    private static final int ZERO = 48;
    private static final int NINE = 57;
    private static final int SMALL_A = 97;
    private static final int SMALL_Z = 122;
    private static final int CAPITAL_A = 65;
    private static final int CAPITAL_Z = 90;
    private static final int MAX_CP_COUNT = 256;
    private static final int UINT_MAGIC = Integer.MIN_VALUE;
    private static final long ULONG_MAGIC = Long.MIN_VALUE;
    static final int[] basicToDigit;
    
    public Punycode() {
    }
    
    private static int adaptBias(int i, int n, boolean b) {
        if (b) {
            i /= 700;
        }
        else {
            i /= 2;
        }
        int n2;
        for (i += i / n, n2 = 0; i > 455; i /= 35, n2 += 36) {}
        return n2 + 36 * i / (i + 38);
    }
    
    private static char asciiCaseMap(char c, boolean b) {
        if (b) {
            if ('a' <= c && c <= 'z') {
                c -= ' ';
            }
        }
        else if ('A' <= c && c <= 'Z') {
            c += ' ';
        }
        return c;
    }
    
    private static char digitToBasic(int n, boolean b) {
        if (n < 26) {
            if (b) {
                return (char)(65 + n);
            }
            return (char)(97 + n);
        }
        else {
            return (char)(22 + n);
        }
    }
    
    public static StringBuffer encode(StringBuffer sb, boolean[] array) throws ParseException {
        int[] array2 = new int[256];
        int length = sb.length();
        int n = 256;
        char[] str = new char[n];
        StringBuffer sb2 = new StringBuffer();
        int n2;
        int i = n2 = 0;
        int j = 0;
        while (j < length) {
            if (n2 == 256) {
                throw new ParseException("Too many input code points", -1);
            }
            int char1 = sb.charAt(j);
            if (isBasic(char1)) {
                if (i < n) {
                    array2[n2++] = 0;
                    str[i] = ((array != null) ? asciiCaseMap((char)char1, array[j]) : char1);
                }
                ++i;
            }
            else {
                int n3 = ((array != null && array[j]) ? 1 : 0) << 31;
                int n4;
                if (!UTF16.isSurrogate((char)char1)) {
                    n4 = (n3 | char1);
                }
                else {
                    int char2;
                    if (UTF16.isLeadSurrogate((char)char1) && j + 1 < length && UTF16.isTrailSurrogate((char)(char2 = sb.charAt(j + 1)))) {
                        ++j;
                        n4 = (n3 | UCharacter.getCodePoint((char)char1, (char)char2));
                    }
                    else {
                        throw new ParseException("Illegal char found", -1);
                    }
                }
                array2[n2++] = n4;
            }
            ++j;
        }
        if ((i = i) > 0) {
            if (i < n) {
                str[i] = '-';
            }
            ++i;
        }
        int n5 = 128;
        int n6 = 0;
        int adaptBias = 72;
        while (i < n2) {
            int n7 = Integer.MAX_VALUE;
            for (int k = 0; k < n2; ++k) {
                int n8 = array2[k] & Integer.MAX_VALUE;
                if (n5 <= n8 && n8 < n7) {
                    n7 = n8;
                }
            }
            if (n7 - n5 > (2147483391 - n6) / (i + 1)) {
                throw new RuntimeException("Internal program error");
            }
            n6 += (n7 - n5) * (i + 1);
            n5 = n7;
            for (int l = 0; l < n2; ++l) {
                int n9 = array2[l] & Integer.MAX_VALUE;
                if (n9 < n5) {
                    ++n6;
                }
                else if (n9 == n5) {
                    int n10 = 36;
                    while (true) {
                        int n11 = n10 - adaptBias;
                        if (n11 < 1) {
                            n11 = 1;
                        }
                        else if (n10 >= adaptBias + 26) {
                            n11 = 26;
                        }
                        if (n6 < n11) {
                            break;
                        }
                        if (i < n) {
                            str[i++] = digitToBasic(n11 + (n6 - n11) % (36 - n11), false);
                        }
                        n6 = (n6 - n11) / (36 - n11);
                        n10 += 36;
                    }
                    if (i < n) {
                        str[i++] = digitToBasic(n6, array2[l] < 0);
                    }
                    adaptBias = adaptBias(n6, i + 1, i == i);
                    n6 = 0;
                    ++i;
                }
            }
            ++n6;
            ++n5;
        }
        return sb2.append(str, 0, i);
    }
    
    private static boolean isBasic(int n) {
        return n < 128;
    }
    
    private static boolean isBasicUpperCase(int n) {
        return 65 <= n && n <= 90;
    }
    
    private static boolean isSurrogate(int n) {
        return (n & 0xFFFFF800) == 0xD800;
    }
    
    public static StringBuffer decode(StringBuffer sb, boolean[] array) throws ParseException {
        int i = sb.length();
        StringBuffer sb2 = new StringBuffer();
        int n = 256;
        char[] str = new char[n];
        while (i > 0 && sb.charAt(--i) != '-') {}
        int n3;
        int len;
        int n2 = len = (n3 = i);
        while (i > 0) {
            int char1 = sb.charAt(--i);
            if (!isBasic(char1)) {
                throw new ParseException("Illegal char found", -1);
            }
            if (i >= n) {
                continue;
            }
            str[i] = char1;
            if (array == null) {
                continue;
            }
            array[i] = isBasicUpperCase(char1);
        }
        int n4 = 128;
        int moveCodePointOffset = 0;
        int adaptBias = 72;
        int n5 = 1000000000;
        int j = (n2 > 0) ? (n2 + 1) : 0;
    Label_0155:
        while (j < i) {
            int n6 = 1;
            int n7 = 36;
            while (j < i) {
                int n8 = Punycode.basicToDigit[(byte)sb.charAt(j++)];
                if (n8 < 0) {
                    throw new ParseException("Invalid char found", -1);
                }
                if (n8 > (Integer.MAX_VALUE - moveCodePointOffset) / n6) {
                    throw new ParseException("Illegal char found", -1);
                }
                moveCodePointOffset += n8 * n6;
                int n9 = n7 - adaptBias;
                if (n9 < 1) {
                    n9 = 1;
                }
                else if (n7 >= adaptBias + 26) {
                    n9 = 26;
                }
                if (n8 < n9) {
                    ++n3;
                    adaptBias = adaptBias(moveCodePointOffset - moveCodePointOffset, n3, moveCodePointOffset == 0);
                    if (moveCodePointOffset / n3 > Integer.MAX_VALUE - n4) {
                        throw new ParseException("Illegal char found", -1);
                    }
                    n4 += moveCodePointOffset / n3;
                    moveCodePointOffset %= n3;
                    if (n4 > 1114111 || isSurrogate(n4)) {
                        throw new ParseException("Illegal char found", -1);
                    }
                    int charCount = UTF16.getCharCount(n4);
                    if (len + charCount < n) {
                        if (moveCodePointOffset <= n5) {
                            if (charCount > 1) {
                                n5 = moveCodePointOffset;
                            }
                            else {
                                ++n5;
                            }
                        }
                        else {
                            moveCodePointOffset = UTF16.moveCodePointOffset(str, 0, len, n5, moveCodePointOffset - n5);
                        }
                        if (moveCodePointOffset < len) {
                            System.arraycopy(str, moveCodePointOffset, str, moveCodePointOffset + charCount, len - moveCodePointOffset);
                            if (array != null) {
                                System.arraycopy(array, moveCodePointOffset, array, moveCodePointOffset + charCount, len - moveCodePointOffset);
                            }
                        }
                        if (charCount == 1) {
                            str[moveCodePointOffset] = (char)n4;
                        }
                        else {
                            str[moveCodePointOffset] = UTF16.getLeadSurrogate(n4);
                            str[moveCodePointOffset + 1] = UTF16.getTrailSurrogate(n4);
                        }
                        if (array != null) {
                            array[moveCodePointOffset] = isBasicUpperCase((int)sb.charAt(j - 1));
                            if (charCount == 2) {
                                array[moveCodePointOffset + 1] = false;
                            }
                        }
                    }
                    len += charCount;
                    ++moveCodePointOffset;
                    continue Label_0155;
                }
                else {
                    if (n6 > Integer.MAX_VALUE / (36 - n9)) {
                        throw new ParseException("Illegal char found", -1);
                    }
                    n6 *= 36 - n9;
                    n7 += 36;
                }
            }
            throw new ParseException("Illegal char found", -1);
        }
        sb2.append(str, 0, len);
        return sb2;
    }
    
    static {
        basicToDigit = new int[] { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, -1, -1, -1, -1, -1, -1, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -1, -1, -1, -1, -1, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 };
    }
}
